import { useMediaQuery } from '@mui/material';
import { useMemo } from 'react';

import { createNuskinTheme } from '../utils';
import { NextGenThemeProviderProps } from './types';

export const useAppTheme = ({ mode, theme, locale }: Pick<NextGenThemeProviderProps, 'mode' | 'theme' | 'locale'>) => {
    const prefersDarkMode = useMediaQuery('(prefers-color-scheme: dark)');

    const isDarkMode = mode ? mode === 'dark' : prefersDarkMode;

    const appMuiTheme = useMemo(() => {
        return createNuskinTheme(theme, isDarkMode, locale) || {};
    }, [isDarkMode, locale, theme]);
    return { appMuiTheme, isDarkMode };
};
