import type { Palette, PaletteKeys } from '@mui/material/styles';

import { ComponentConfig } from '.';
import { GenomeColorType } from '../genome-colors/AllColors';

export type PaletteThemeColors =
    | 'primary'
    | 'neutral'
    | 'accentColorA'
    | 'accentColorB'
    | 'accentColorC'
    | 'accentColorD'
    | 'secondary'
    | 'success'
    | 'warning'
    | 'error'
    | 'info';
export const PaletteColors: PaletteKeys[] = [
    'primary',
    'neutral',
    'accentColorA',
    'accentColorB',
    'accentColorC',
    'accentColorD',
    'secondary',
    'success',
    'warning',
    'error',
    'info',
];

export const AppColorGroups = [...PaletteColors, 'custom'];

export namespace FoundationTheme {
    export type Mode = 'light' | 'dark';

    export type HexColor = `#${string}`;

    export type CommonColors = {
        white: string;
        black: string;
    };

    type ColorGroupPrimary = {
        primary?: {
            black: HexColor;
            white: HexColor;
            gray: HexColor;
            dark: HexColor;
        };
    };

    export type PaletteColors = keyof Palette;

    export type AppColors = (typeof AppColorGroups)[number];

    export type MainDarkColors = { main: string; dark?: string };

    export type Colors = Record<AppColors, Record<string, string | MainDarkColors> & ColorGroupPrimary>;

    export type TypographyVariants =
        | 'h1'
        | 'h2'
        | 'h3'
        | 'h4'
        | 'h5'
        | 'h6'
        | 'title-l'
        | 'title-m'
        | 'title-s'
        | 'label-l'
        | 'label-m'
        | 'label-s'
        | 'label-l'
        | 'label-m'
        | 'label-s'
        | 'body-xl'
        | 'body-l'
        | 'body-m'
        | 'body-s'
        | 'body-xs'
        | 'CTA';

    export type FontSections =
        | 'headings'
        | 'subtitles'
        | 'body'
        | 'labels'
        | 'buttons'
        | 'captions'
        | 'overlines'
        | 'title';

    export type FontConfig = {
        type: 'GoogleFont' | 'LocalFont';
        fontName: string;
        /*
            Font Style - Denotes [weight, italics] pairs
            Example = [[400, 0], [600, 0], [700, 1]
        */
        fontStyle: ([number, number] | number)[];
    };

    export interface ButtonState {
        hovered?: string;
        hoveredText?: string;
        pressed?: string;
        pressedText?: string;
        focused?: string;
        focusedText?: string;
        disabled?: string;
        disabledText?: string;
    }

    export type PaletteColorType = {
        main: string;
        contrastText: string;
        buttonState?: ButtonState;
        darkMode?: {
            main: string;
            contrastText: string;
            buttonState?: ButtonState;
        };
    };

    export type ThemeConfig = {
        components?: ComponentConfig;
    };

    export type Theme = {
        palette?: Palette;
        colors?: Colors;
        config?: ThemeConfig;
        fonts?: Record<
            string,
            {
                defaultFont: string;
                fontsToLoad: FontConfig[];
                section: Record<FontSections, string>;
                typographyVariants: Record<
                    TypographyVariants,
                    {
                        fontFamily: string;
                        fontSize?: number;
                        lineHeight?: number;
                        letterSpacingPercentage?: number;
                        paragraphSpacing?: number;
                        fontWeight?: number;
                    }
                >;
            }
        >;
        genomeColors?: GenomeColorType;
        newPalette?: GenomeColorType;
    };

    export type Themed = { theme: FoundationTheme.Theme };
}
