import { FoundationTheme } from '../types/foundationTheme';
import { AllColorKeyType, AllColors } from './AllColors';

/**
 * first we try to get we can get the color code by using theme.newPalette
 * (new Palette will be replaced with palette when old implementation of colorUtils.color is deprecated)
 * if newPalette does not exist then we try to fetch color from the AllColors
 * if color value is not present in AllColors as well then we return a wrong value '#ggg'
 * @param theme object coming from material ui
 * @param colorName example: N100, N10, Success-100, 'Lumi-Primary-A'
 * @param forceMode 'light' | 'dark' | undefined
 * @returns hex value for the color
 */
export const getGenomeColor = (
    theme: FoundationTheme.Theme,
    colorName: AllColorKeyType,
    forceMode?: 'light' | 'dark',
) => {
    const mode = (forceMode || theme?.palette?.mode) ?? 'light';

    let colors = theme?.newPalette;

    let colorValue = colors?.[colorName]?.[mode]??'';
    if (!colorValue) {
        const colorObj = AllColors[colorName];
        colorValue = colorObj?.[mode];
    }
    if (colorValue) {
        return colorValue;
    } else {
        console.error('No color found for colorName = ' + colorName);
        return '#ggg';
    }
};
